//
//  DNNode.h
//  dNodi
//
//  Created by Derek Clarkson on 7/17/11.
//  Copyright 2011. All rights reserved.
//

/**
 DNNode is a protocol which represents the connection between dNodi and your data structure. Classes which implement this protocol can be used by dNodi to execute it's selectors and filters against.
 */
@protocol DNNode <NSObject>

/**
 Returns the name of the node.
 */
@property(readonly) NSString *dnName;

/**
 Returns a reference to the parent node of this node.
 */
@property(readonly) id<DNNode> dnParentNode;

/**
 Returns an array of the subnodes of this node. You can return either an empty NSArray or nil.
 */
@property(readonly) NSArray *dnSubNodes;

/**
 Asks the node if it has a specific attribute.
 
 This design allows the node to decide how it will respond to particular attribute.
 
 @param attribute the attribute name
 @param value the value we want to compare. It is up to the implementor how this is done.
 @return YES if there is an attribute with a matching value.
 */
-(BOOL) dnHasAttribute:(NSString *) attribute withValue:(NSString *) value;

@end
