//
//  DNNodeRule.m
//  dXml
//
//  Created by Derek Clarkson on 26/01/10.
//  Copyright 2010 Derek Clarkson. All rights reserved.
//

#import <dNodi/DNNodeNameSelector.h>
#import <dUsefulStuff/DCCommon.h>

@interface DNNodeNameSelector () {
	NSRegularExpression *_regex;
}

@end

@implementation DNNodeNameSelector

@synthesize nodeName = _nodeName;

- (void) dealloc {
	DC_DEALLOC(_regex);
	DC_DEALLOC(_nodeName);
	[super dealloc];
}

- (id) initWithNodeName:(NSString *)nodeName error:(NSError **)error {
	self = [super init];
	if (self) {
		if ([nodeName rangeOfString:@"*"].location != NSNotFound) {
			NSString *pattern = [NSString stringWithFormat:@"^%@$", [nodeName stringByReplacingOccurrencesOfString: @"*" withString:@".*"]];
			_regex = [[NSRegularExpression alloc] initWithPattern:pattern options:0 error:error];
			if (_regex == nil) {
				[self release];
				return nil;
			}
		}
		_nodeName = [nodeName retain];
	}
	return self;
}

-(NSArray *) processNode:(id<DNNode>)node index:(NSUInteger)index {
	if (_regex == nil) {
		DC_LOG(@"Checking '%@', matches '%@'?: %@", node.dnName, _nodeName, [node.dnName isEqualToString:_nodeName] ? @"Yes" : @"No");
		return [node.dnName isEqualToString:_nodeName] ? @[node] : nil;
	}
	NSRange match = [_regex rangeOfFirstMatchInString:node.dnName options:0 range:NSMakeRange(0, [node.dnName length])];
	DC_LOG(@"Checking '%@', regex based match with '%@': %@", node.dnName, _regex.pattern, match.location == NSNotFound ? @"No" : @"Yes");
	return match.location == NSNotFound ? nil : @[node];
}

@end
