//
//  DNParser.h
//  dNodi
//
//  Created by Derek Clarkson on 7/22/11.
//  Copyright 2011. All rights reserved.
//

#import <dNodi/DNSelectorFactory.h>
#import <dNodi/DNSelectorFactoryDelegate.h>
#import <dNodi/DNSelector.h>
#import <dNodi/DNScannerDelegate.h>

/**
 Parser which analyses a query and generates an array of selectors which can then be run against the nodes.
 */
@interface DNParser : NSObject<DNSelectorFactoryDelegate, DNScannerDelegate>

/**
 Gives access to the selector factory. Mainly for convenience and testing.
 */
@property (retain, nonatomic) DNSelectorFactory *selectorFactory;

/**
 Analyses the passed query and generates a list of selectors.

 @param query the query to process.
 @param error a reference to an NSError variable that will be populated if there is any problem. In this case the returned array will be nil.
 @return an array of the found nodes. Or nil if there is an error in the error reference.
 */
-(id<DNSelector>) selectorChainForQuery:(NSString *) query error:(NSError **) error;

@end
