//
//  DNScannerDelegate.h
//  dNodi
//
//  Created by Derek Clarkson on 23/11/12.
//
//

#import <dNodi/DNCommon.h>

/**
 Protocol implemented by classes needing to receive tokens and data from scanning a xpath text string.
 */
 
@protocol DNScannerDelegate <NSObject>

/**
 Called each time the scanner reads some data. 
 
 The job of the delelgate is to process the recevied token. If this triggers an error then the error reference should be populated and NO returned.
 
 @param token the token just read.
 @param location the index in the path at which the scanner started read this token.
 @param value if a value was read for the token, this contains it.
 @param error a pointer to an error variable which can be populated by the delegate should something not be right.
 @result return YES if everything is ok and the token accepted. NO if there is a problem with the token. error should be populated.
 */
-(BOOL) didScanToken:(DNToken) token
			 atLocation:(NSUInteger) location
			  withValue:(id) value
					error:(NSError **) error;

/**
 Called after all tokens have been read.
 
 But not called if there was a prior error.
 
 @param error a pointer to an error variable which can be populated by the delegate should something not be right.
 @result return YES if everything is ok and the token accepted. NO if there is a problem with the token. error should be populated.
 */
-(BOOL) scanDidEndWithError:(NSError **) error;

@end
