//
//  DNSelector.h
//  dNodi
//
//  Created by Derek Clarkson on 18/11/12.
//
//

/**
 Protocol that all selectors must implement.
 
 @see DNAbstractSelector
 */

@protocol DNSelector <NSObject>

/// @name Properties

/// Set to the next selector in the chain. Each selector should call the next once it has finished processing the current array of nodes and has a new array to pass to it.
@property (nonatomic, retain) id<DNSelector> nextSelector;

/// @name Tasks

/**
 Runs the selector against the passed in nodes.  

 @param nodes a list of the nodes to check. Each node must implement the DBNode protocol.
 @return a NSArray of nodes. If empty, no nodes where selected.
 */
-(NSArray *) executeWithNodes:(NSArray *) nodes;

@end
