//
//  DNSelectorFactory.h
//  dNodi
//
//  Created by Derek Clarkson on 25/07/11.
//  Copyright 2011. All rights reserved.
//

#import <dNodi/DNSelectorFactoryDelegate.h>
#import <dNodi/DNCommon.h>

/**
 Factory class which receives instructions from DNParser and constructs a list of selectors to execute.
 */
@interface DNSelectorFactory : NSObject

/// @name Initialisers

/**
 Default initialiser which takes a delegate.
 
 @param delegate the object that will be told when selectors are created.
 @result an instance of this class.
 */
-(id) initWithSelectorFactoryDelegate:(id<DNSelectorFactoryDelegate>) delegate;

/// @name Tasks

/**
 Called to pass a read token across.
 
 @param token the token enum that was just read.
 @param value the value associated with the token or nil if there is none.
 @param error a reference to an NSError variable which is populated if an error occurs.
 @return YES if the token was processed ok, NO if tehre was an error.
 */
-(BOOL) tokenRead:(DNToken) token value:(id) value error:(NSError **) error;

@end
