//
//  DNSelectorFactory.m
//  dNodi
//
//  Created by Derek Clarkson on 25/07/11.
//  Copyright 2011. All rights reserved.
//

#import <dNodi/DNSelectorFactory.h>
#import <dUsefulStuff/DCCommon.h>

#import <dNodi/DNAbstractSelector.h>
#import <dNodi/DNNodeNameSelector.h>
#import <dNodi/DNIndexedSelector.h>
#import <dNodi/DNChildNodesSelector.h>
#import <dNodi/DNTreeSearchSelector.h>
#import <dNodi/DNParentSelector.h>
#import <dNodi/DNAttributeSelector.h>

@interface DNSelectorFactory() {
@private
	NSMutableString *_attributeName;
	id<DNSelectorFactoryDelegate> _delegate;
}

@end

@implementation DNSelectorFactory

#pragma mark - Lifecycle

-(void) dealloc {
	DC_DEALLOC(_attributeName);
	[super dealloc];
}

-(id) initWithSelectorFactoryDelegate:(id<DNSelectorFactoryDelegate>) delegate {
	self = [super init];
	if (self) {
		_delegate = delegate; // weak.
		_attributeName = nil;
	}
	return self;
}

-(BOOL) tokenRead:(DNToken) token value:(id) value error:(NSError **) error {
	
	id<DNSelector> selector = nil;
	BOOL success = YES;
	switch (token) {
			
		case DNTokenSlash:
			DC_LOG(@"Sub nodes requested. Adding subnodes selector.");
			selector = [[DNChildNodesSelector alloc] init];
			break;
			
		case DNTokenNodeName:
			DC_LOG(@"Adding node selector for \"%@\"", value);
			selector = [[DNNodeNameSelector alloc] initWithNodeName:value error:error];
			if (selector == nil) {
				success = NO;
			}
			break;
			
		case DNTokenNumber: {
			NSUInteger number = [(NSNumber *) value intValue];
			DC_LOG(@"Adding index only selector for index %i", number);
			selector = [[DNIndexedSelector alloc] initWithIndex:number];
			break;
		}
			
		case DNTokenParent:
			DC_LOG(@"Adding parent node selector.");
			selector = [[DNParentSelector alloc] init];
			break;

		case DNTokenComma:
		case DNTokenOpenBracket:
			// Get read for a selector name.
			_attributeName = [[NSMutableString alloc] init];
			break;
			
		case DNTokenAttributeName:
			DC_LOG(@"Storing attribute name %@", (NSString *) value);
			[_attributeName appendString:value];
			break;
			
		case DNTokenProperty:
			[_attributeName appendString:@"."];
			break;
			
		case DNTokenValue: {
			DC_LOG(@"Adding attribute selector for attribute %@='%@'", _attributeName, value);
			selector = [[DNAttributeSelector alloc] initWithAttributeName:_attributeName value:value];
			DC_DEALLOC(_attributeName);
			break;
		}
			
		case DNTokenSubtree:
			DC_LOG(@"Adding subtree selector.");
			selector = [[DNTreeSearchSelector alloc] init];
			break;
			
		default:
			break;
	}
	
	// If a selector has been produced, send it to the delegate.
	if (selector != nil) {
		[_delegate didCreateSelector:selector];
		DC_DEALLOC(selector);
	}
	return success;
}


@end
