//
//  Created by Derek Clarkson on 21/01/10.
//  Copyright 2010 Derek Clarkson. All rights reserved.
//
#import <GHUnitIOS/GHUnit.h>
#import <dUsefulStuff/DCCommon.h>

#import <dNodi/DNExecutor.h>
#import "AbstractTestBase.h"
#import <dNodi/dNodi.h>
#import "DummyNode.h"

@interface DNExecutorTests : AbstractTestBase {
@private
	DNExecutor *executor;
}
@end

@implementation DNExecutorTests

- (void) setUp {
	[super setUp];
	executor = [[DNExecutor alloc] initWithRootNode:abc];
	error = nil;
}

-(void) tearDown {
	DC_DEALLOC(executor);
}

- (void) testInvalidSyntaxTriggersError {
	id results = [executor executeQuery:@"/[[" error:&error];
	GHAssertNil(results, nil);
	GHAssertNotNil(error, nil);
	GHAssertEqualStrings([error domain], DN_ERROR_DOMAIN, nil);
	GHAssertEquals([error code], DNErrorInvalidNodiExpression, nil);
	GHAssertEqualStrings([error domain], DN_ERROR_DOMAIN, nil);
}

- (void) testPathRunsOutOfNodesReturnsEmptyList {
	NSArray *results = [executor executeQuery:@"//x" error:&error];
	GHAssertEquals([results count], (NSUInteger) 0, nil);
}

- (void) testIncorrectRootNodeReferenceReturnsNSNull {
	NSArray *results = [executor executeQuery:@"/xyz" error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 0, nil);
}

- (void) testTooLongPathReturnsNSNull {
	NSArray *results = [executor executeQuery:@"//abc/def/ghi/lmn/opq" error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 0, nil);
}

- (void) testParentReference {
	executor.rootNode = lmn; 
	NSArray *results = [executor executeQuery:@"/.." error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(results[0], def1, nil);
}

- (void) testNodeNameArray {
	NSArray *results = [executor executeQuery:@"/def" error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals((int)[results count], 2, nil);
	for (NSObject<DNNode> *node in results) {
		GHAssertEqualStrings(node.dnName, @"def", nil);
	}
}

- (void) testArrayIndex {
	NSArray *results = [executor executeQuery:@"/[0]" error:&error];	
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(((id<DNNode>)results[0]).dnName, @"def", nil);	
}

- (void) testNodeNameAndArrayIndex {	
	NSArray *results = [executor executeQuery:@"/def[1]" error:&error];	
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(results[0], def1, nil);
}

- (void) testNodeNameAndSubNodeArrayIndex {	
	NSArray *results = [executor executeQuery:@"/def[1]/[0]" error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(((id<DNNode>)results[0]).dnName, @"lmn", nil);
	
}

- (void) testSubtreeFindsNode {	
	NSArray *results = [executor executeQuery:@"//lmn" error:&error];	
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(((id<DNNode>)results[0]).dnName, @"lmn", nil);	
}

- (void) testSubtreeFindsNthNode {	
	NSArray *results = [executor executeQuery:@"//def[1]" error:&error];	
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(((id<DNNode>)results[0]).dnName, @"def", nil);
	NSObject<DNNode> *subNode = [((id<DNNode>)results[0]).dnSubNodes objectAtIndex:0];
	GHAssertEqualStrings(subNode.dnName, @"lmn", nil);	
}

- (void) testNodeAndAttribute {	
	NSArray *results = [executor executeQuery:@"/def[xyz='xxx']" error:&error];	
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(((id<DNNode>)results[0]).dnName, @"def", nil);
}

- (void) testNodeAndPropertyAttribute {
	NSArray *results = [executor executeQuery:@"/opq[abc.def='qqq']" error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEquals(results[0], opq, nil);
}

- (void) testNodeAndMultipleAttributes {
	NSArray *results = [executor executeQuery:@"/def[xyz='xxx',zyx='yyy']" error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(results[0], def0, nil);
}

- (void) testSubtreeSearchThenParent {
	NSArray *results = [executor executeQuery:@"//ghi/.." error:&error];	
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEquals(results[0], def0, nil);
}

- (void) testSubtreeSearchWithAttribute {
	NSArray *results = [executor executeQuery:@"//[zyx='yyy']" error:&error];
	GHAssertNil(error, nil);
	GHAssertEquals([results count], (NSUInteger) 1, nil);
	GHAssertEquals(results[0], def0, nil);
}

@end
