//
//  DNRootNodeRuleTests.m
//  dXml
//
//  Created by Derek Clarkson on 25/01/10.
//  Copyright 2010 Derek Clarkson. All rights reserved.
//
#import "AbstractTestBase.h"
#import <dUsefulStuff/DCCommon.h>
#import <dNodi/DNNodeNameSelector.h>
#import <dNodi/DNSelector.h>
#import "DummyNode.h"

@interface DNNodeSelectorTests : AbstractTestBase

@end

@implementation DNNodeSelectorTests

-(void) testInitRegectsRegex {
	id<DNSelector> selector = [[[DNNodeNameSelector alloc] initWithNodeName:@"[.*" error: &error] autorelease];
	GHAssertNil(selector, nil);
	GHAssertTrue(error != NULL, nil);
}

- (void) testBasicSelection {
	id<DNSelector> selector = [[[DNNodeNameSelector alloc] initWithNodeName:@"abc" error: &error] autorelease];
	GHAssertTrue(error == NULL, nil);
	NSArray *nodes = [selector executeWithNodes:@[abc]];
	GHAssertEquals([nodes count], (NSUInteger) 1, nil);
	GHAssertEqualStrings(((DummyNode *)nodes[0]).dnName, @"abc", nil);
}

- (void) testSelectsMultipleNodes {
	id<DNSelector> selector = [[[DNNodeNameSelector alloc] initWithNodeName:@"def" error: &error] autorelease];
	GHAssertTrue(error == NULL, nil);
	NSArray *nodes = [selector executeWithNodes:abc.dnSubNodes];
	GHAssertEquals([nodes count], (NSUInteger) 2, nil);
	GHAssertEqualStrings(((DummyNode *)nodes[0]).dnName, @"def", nil);
	GHAssertEqualStrings(((DummyNode *)nodes[1]).dnName, @"def", nil);
	GHAssertNotEquals(nodes[0], nodes[1], nil);
}

- (void) testSelectsWildcardPrefix {
	id<DNSelector> selector = [[[DNNodeNameSelector alloc] initWithNodeName:@"*pq" error: &error] autorelease];
	GHAssertTrue(error == NULL, nil);
	NSArray *nodes = [selector executeWithNodes:abc.dnSubNodes];
	GHAssertEquals([nodes count], (NSUInteger) 2, nil);
	GHAssertEqualStrings(((DummyNode *)nodes[0]).dnName, @"opq", nil);
	GHAssertEqualStrings(((DummyNode *)nodes[1]).dnName, @"zpq", nil);
}

- (void) testSelectsWildcardSuffix {
	id<DNSelector> selector = [[[DNNodeNameSelector alloc] initWithNodeName:@"op*" error: &error] autorelease];
	GHAssertTrue(error == NULL, nil);
	NSArray *nodes = [selector executeWithNodes:abc.dnSubNodes];
	GHAssertEquals([nodes count], (NSUInteger) 2, nil);
	GHAssertEqualStrings(((DummyNode *)nodes[0]).dnName, @"opq", nil);
	GHAssertEqualStrings(((DummyNode *)nodes[1]).dnName, @"opz", nil);
}

- (void) testSelectsWildcardInfix {
	id<DNSelector> selector = [[[DNNodeNameSelector alloc] initWithNodeName:@"o*q" error: &error] autorelease];
	GHAssertTrue(error == NULL, nil);
	NSArray *nodes = [selector executeWithNodes:abc.dnSubNodes];
	GHAssertEquals([nodes count], (NSUInteger) 2, nil);
	GHAssertEqualStrings(((DummyNode *)nodes[0]).dnName, @"opq", nil);
	GHAssertEqualStrings(((DummyNode *)nodes[1]).dnName, @"ozq", nil);
}

- (void) testSelectsWildcardOuters {
	id<DNSelector> selector = [[[DNNodeNameSelector alloc] initWithNodeName:@"*p*" error: &error] autorelease];
	GHAssertTrue(error == NULL, nil);
	NSArray *nodes = [selector executeWithNodes:abc.dnSubNodes];
	GHAssertEquals([nodes count], (NSUInteger) 3, nil);
	GHAssertEqualStrings(((DummyNode *)nodes[0]).dnName, @"opq", nil);
	GHAssertEqualStrings(((DummyNode *)nodes[1]).dnName, @"opz", nil);
	GHAssertEqualStrings(((DummyNode *)nodes[2]).dnName, @"zpq", nil);
}

@end
