//
//  DNSelectorFactoryTests.m
//  dNodi
//
//  Created by Derek Clarkson on 26/07/11.
//  Copyright 2011. All rights reserved.
//

#import <GHUnitIOS/GHUnit.h>
#import <dUsefulStuff/DCCommon.h>

#import <dNodi/DNSelectorFactory.h>

#import <dNodi/DNNodeNameSelector.h>
#import <dNodi/DNIndexedSelector.h>
#import <dNodi/DNParentSelector.h>
#import <dNodi/DNAttributeSelector.h>
#import <dNodi/DNChildNodesSelector.h>
#import <dNodi/DNTreeSearchSelector.h>

@interface DNSelectorFactoryTests : GHTestCase<DNSelectorFactoryDelegate> {
@private
	DNSelectorFactory *factory;
	id<DNSelector> _firstSelector;
	id<DNSelector> _lastSelector;
	NSError *error;
}
@end

@implementation DNSelectorFactoryTests

-(void) setUp {
	error = NULL;
	factory = [[DNSelectorFactory alloc] initWithSelectorFactoryDelegate:self];
}

-(void) tearDown {
	DC_DEALLOC(_firstSelector);
	DC_DEALLOC(_lastSelector);
	DC_DEALLOC(factory);
}

-(void) testSubnodesSelector {
	GHAssertTrue([factory tokenRead:DNTokenSlash value:nil error:&error], nil);
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue([_firstSelector isKindOfClass:[DNChildNodesSelector class]], nil);
}

-(void) testParentSelector {
	GHAssertTrue([factory tokenRead:DNTokenParent value:nil error:&error], nil);
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue([_firstSelector isKindOfClass:[DNParentSelector class]], nil);
}


-(void) testNodeNameSelector {
	GHAssertTrue([factory tokenRead:DNTokenNodeName value:@"abc" error:&error], nil);
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue([_firstSelector isKindOfClass:[DNNodeNameSelector class]], nil);
	DNNodeNameSelector *selector = (DNNodeNameSelector *) _firstSelector;
	GHAssertEqualStrings(selector.nodeName, @"abc", nil);
}

-(void) testSubtreeSelectorAfterName {
	GHAssertTrue([factory tokenRead:DNTokenSubtree value:nil error:&error], nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(_lastSelector != NULL, nil);
	GHAssertTrue([_lastSelector isKindOfClass:[DNTreeSearchSelector class]], nil);
}

-(void) testIndexSelector {
	[factory tokenRead:DNTokenNumber value:@5 error:&error];
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue([_firstSelector isKindOfClass:[DNIndexedSelector class]], nil);
}

-(void) testOpenBracketDoesNothing {
	[factory tokenRead:DNTokenOpenBracket value:nil error:&error];
	GHAssertNil(_firstSelector, @"Should be zero selectors");
}

-(void) testCloseBracketDoesNothing {
	[factory tokenRead:DNTokenCloseBracket value:nil error:&error];
	GHAssertNil(_firstSelector, @"Should be zero selectors");
}

-(void) testAttributeSelector {
	
	[factory tokenRead:DNTokenOpenBracket value:nil error:&error];
	[factory tokenRead:DNTokenAttributeName value:@"def" error:&error];
	[factory tokenRead:DNTokenEquals value:nil error:&error];
	[factory tokenRead:DNTokenOpenQuote value:nil error:&error];
	[factory tokenRead:DNTokenValue value:@"ghi" error:&error];
	[factory tokenRead:DNTokenCloseQuote value:nil error:&error];
	[factory tokenRead:DNTokenCloseBracket value:nil error:&error];
	
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue([_firstSelector isKindOfClass:[DNAttributeSelector class]], nil);
	DNAttributeSelector *selector = (DNAttributeSelector *) _firstSelector;
	GHAssertEqualStrings(selector.attributeName, @"def", nil);
	GHAssertEqualStrings(selector.value, @"ghi", nil);
}

-(void) testAttributePropertySelector {
	
	[factory tokenRead:DNTokenOpenBracket value:nil error:&error];
	[factory tokenRead:DNTokenAttributeName value:@"abc" error:&error];
	[factory tokenRead:DNTokenProperty value:nil error:&error];
	[factory tokenRead:DNTokenAttributeName value:@"def" error:&error];
	[factory tokenRead:DNTokenEquals value:nil error:&error];
	[factory tokenRead:DNTokenOpenQuote value:nil error:&error];
	[factory tokenRead:DNTokenValue value:@"ghi" error:&error];
	[factory tokenRead:DNTokenCloseQuote value:nil error:&error];
	[factory tokenRead:DNTokenCloseBracket value:nil error:&error];
	
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue([_firstSelector isKindOfClass:[DNAttributeSelector class]], nil);
	DNAttributeSelector *selector = (DNAttributeSelector *) _firstSelector;
	GHAssertEqualStrings(selector.attributeName, @"abc.def", nil);
	GHAssertEqualStrings(selector.value, @"ghi", nil);
	GHAssertNil(selector.nextSelector, nil);
}

-(void) testMultipleAttributesCreateMultipleSelectors {
	
	[factory tokenRead:DNTokenOpenBracket value:nil error:&error];
	
	[factory tokenRead:DNTokenAttributeName value:@"abc" error:&error];
	[factory tokenRead:DNTokenEquals value:nil error:&error];
	[factory tokenRead:DNTokenOpenQuote value:nil error:&error];
	[factory tokenRead:DNTokenValue value:@"def" error:&error];
	[factory tokenRead:DNTokenCloseQuote value:nil error:&error];
	
	[factory tokenRead:DNTokenComma value:nil error:&error];
	
	[factory tokenRead:DNTokenAttributeName value:@"def" error:&error];
	[factory tokenRead:DNTokenEquals value:nil error:&error];
	[factory tokenRead:DNTokenOpenQuote value:nil error:&error];
	[factory tokenRead:DNTokenValue value:@"abc" error:&error];
	[factory tokenRead:DNTokenCloseQuote value:nil error:&error];
	
	[factory tokenRead:DNTokenCloseBracket value:nil error:&error];
	
	GHAssertTrue(_firstSelector != NULL, nil);
	GHAssertTrue(error == NULL, nil);
	GHAssertTrue([_firstSelector isKindOfClass:[DNAttributeSelector class]], nil);
	DNAttributeSelector *selector = (DNAttributeSelector *) _firstSelector;
	GHAssertEqualStrings(selector.attributeName, @"abc", nil);
	GHAssertEqualStrings(selector.value, @"def", nil);
	selector = (DNAttributeSelector *) _lastSelector;
	GHAssertEqualStrings(selector.attributeName, @"def", nil);
	GHAssertEqualStrings(selector.value, @"abc", nil);
	
}

-(void) didCreateSelector:(id<DNSelector>)selector {
	if (_firstSelector == nil) {
		_firstSelector = [selector retain];
	}
	DC_DEALLOC(_lastSelector);
	_lastSelector = [selector retain];
}

@end
